clear, clc, close all

% This code generates network plots of Fig. 5A for the ChETA variant 
% without buffering (right panel). Please note that due to stochasticity,
% the traces will not exactly match those in the paper. Please also note that 
% this code takes a long time to compute. For faster results either 
% reduce the simulation time or the size of the network.

%% add required paths
addpath(genpath(fullfile(pwd,'../..')))

%% stimulation protocol
wiener_noise = true;
IP3_stim = false;
singlepulse_stim = false;
laser_stim = true;
buffer = false;

%% simulation time
t0 = 0;     %[s] start time
tf = 600;   %[s] end time
dt = 0.1;   %[s] time increment (seems like this is the largest dt that converges)
tspan = t0:dt:tf;    %[s] time span

%% geometry
cells_x = 10;       % cells along the x direction
cells_y = 10;       % cells in the y direction

%% laser stimulus
laser_on = 30;   %[s] stim onset
laser_off = tf;  %[s] stim end
T = 4;           %[s] pulse period
delta =  0.45*T;    %[s] pulse width
laser_stim_cell_indx = [34:37, 44:47, 54:57, 64:67];    % index of stimulated cells

%% make graph for geometry

tot_cell_number = cells_x*cells_y; % network of astrocytes

% --- find adjacency matrix
Adj = zeros(tot_cell_number);
a = ones(1,length(diag(Adj,1)));
a(cells_x:cells_x:numel(a)) = 0;
Adj = diag(a,1);
b = ones(1,length(diag(Adj,cells_x)));
Adj = Adj + diag(b,cells_x);
Adj = Adj+triu(Adj,1).';

%% uncomment to have diagonal connections as well
% a = ones(1,(cells_x*(cells_x-1))+1);
% a(1:cells_x:end) = 0;
% b = ones(1,(cells_x*(cells_x-1))-1);
% b(cells_x:cells_x:end) = 0;
% d1 = diag(a,cells_x-1);
% d2 = diag(b,cells_x+1);
% 
% A = d1 + d2;
% 
% Adj = Adj + A + A';

%% plot network
Gobj = graph(Adj);
figure, plot(Gobj)

%% find neighbors
nnod = numnodes(Gobj);

edgelist = Gobj.Edges{:,1};
neighbors = zeros(nnod, max(Gobj.degree));
for i = 1:nnod
    neighborNodes = Gobj.neighbors(i);
    neighborNodes = neighborNodes';
    neighbors(i,1:numel(neighborNodes)) = neighborNodes;
end

%% IP3 stimulus
IP3_stim_onset = 200;    %[s] onset of the IP3 stimulus
IP3_stim_end = 500;      %[s] end of IP3 stimulus
IP3_level_basal = 0.14;  % uM/s (Independent Variable, Resting = 0.01)
IP3_level_stim = 0.4;    % factor of increase in IP3 level
IP3_stim_cell_indx = [34:37, 44:47, 54:57, 64:67];

%% ChR2 parameters (CHETA)

fact = 1;       % adjust light intesity (1 == control)
GChR2 = 0.8759;  %[mS/cm2]
p1 = fact*0.0661e3;    %s^-1; Max excitation rate of the 1st closed state
gd1 = 0.0102e3;   %s^-1; Rate const. for the o1-> c1 transition state
e12 = 10.5128e3;  %s^-1; Rate const for the o1->o2 transition
e21 = 0.005e3;    %s^-1; Rate const for the o2->o1 transition
p2 = fact*0.0641e3;    %s^-1; Max excitation rate of the 2nd closed state
gd2 = 0.1510e3;   %s^-1; Rate const for the o3-> c2 transition
gr = 1e-003*1e3;  %s^-1; Recovery rate of the 1st closed state after light stimu is turned off
tau = 1.5855e-3;  %activation time of the ChR2 ion channel; gr = 1/tau
gamma = 0.0141;   %g2/g1 (ratio of conductances)
astro_area = 4.83e-6; %[cm^2] area of the astrocyte
Vm = -70;            %membrane voltage (mV)
vol_cyt = 1e-12; %[L] volume of the cytosol
F = 96485.3;    %[C/mol] faraday's constant
zCa = 2;    % valence of calcium
EChR2 = 0;  %[mV] reversal potential of ChR2 (Williams et. al, 2013)
conversion_Fact = 1e-6/(zCa*F*vol_cyt);      % conversion factor for nA to uM/s
G_V = ((10.6408 - (14.6408*exp(-Vm/42.7671)))/Vm);
g1 = GChR2*astro_area*1e6;      % [nS] whole cell conductance of ChR2 channels in the cell (if current is in nA)

%% IP3 dynamics
vdelta = 0.152; % uM/s, Max. rate of IP3 production (PLC); Riera et al, Page 459,
KdCa = 0.55;    % uM; Half saturation const; Values: Fitting by Jorge Riera (PLCd)
KIP3 = 1.25;    % s^-1; IP3 degradation rate; Riera et al, 2011, Supp material for the SCOs in normal/ Alzheimers paper

%% Ca_C dynamics
xCCE = 0.01;    %uM/s; Max. rate of activation dependent Ca2+ influx (CCE); phenomological value; Riera et al - Page 450
hCCE = 10;      % uM; Half-inactivation constant for CCE influx; phenomological value; Riera et al - Page 450
c1 = 0.185;     % addimensional; Volume rate between ER and cytosol-> Ver/Vcyt; Values: 0.028-> DiGarbo, 0.05->Hofel
V_SERCA = 0.9;  % uM/sec; Max. rate const of SERCA pump (Max. Ca2+ pump flux); Riera et al - Page 447 - - check table for more value ranges Values = {0.9 uM/sec -> Goto&Natsume and Stamatakis&Mantzaris; 0.49 uM/sec -> Bennett}
Kp =  0.1;      % uM; Ca2+ sensitivity of the SERCA pump/ Dissociation const. of Ca2+ to SERCA; Riera et al - Page 447 - - check table for more value ranges; Values = {0.1 uM -> Goto&Natsume and Lavrentovich&Hemkin ;  0.05 uM Stamatakis&Mantzaris; 0.24 uM -> Bennett}
d1 =  0.13;     % uM;Dissocation constant for IP3(IP3R); Values = {0.13(d1),0.9434(d3)uM -> Goto&Natsume and Stamatakis&Mantzaris; 0.03uM -> Bennett}
d5 =  0.082;    %uM; Ca2+ activation constant (IP3R); Riera et al, 2011, Supp material for the SCOs in normal/ Alzheimers paper
v1 =  6;        %sec^(-1); Ca2+ channel flux const; Values = {6 sec^(-1) -> Goto&Natsume and Stamatakis&Mantzaris; 0.95 sec^(-1) -> Bennett
v2 = 0.11;      %sec^(-1); Ca2+ leak flux const; Goto&Natsume and Stamatakis&Mantzaris;
kout = 0.5;     %s^-1, Rate constant of Ca2+ extrusion; Riera et al, Page 451
lambda =  1;    % Riera et al SCO paper
epsilon = 0.01; %dimensionless, The ratio of plasma membrane and ER membrane areas (Li & Rinzel, page 468) e_m=0.01 Riera et al, 2011, Supp material for the SCOs in normal/ Alzheimers paper
Jin_Astro = 0.035;    % leak; uM/s, Riera et al, SCO paper-> Pg 449, used a heuristic strategy to determine jin -> 0:036 uM/s  for healthy astrocytes in acute slice preparations from adult C57BL/6 mice.

%% Gating parameters
a = 0.2;        %(uMs)^-1, Rate const for Ca2+ binding in IP3 inhibitory site; Riera et al, 2011, Supp material for the SCOs in normal/ Alzheimers paper
d2 = 1.049;     % uM; Dissociation const for Ca2+ inhibition (IP3R); Riera et al, 2011, Supp material for the SCOs in normal/ Alzheimers paper
d3 = 0.9434;    % uM; Dissociation const for IP3 (IP3R); Riera et al, 2011, Supp material for the SCOs in normal/ Alzheimers paper

%% buffering parameters
bt = 0; % uM
K = 20; % uM

if buffer
    bt = 200; % uM
end

%% Gap junctional diffusion
DIP3 = 1; %[s-1] rate of IP3 diffusion
DCa = 0.1*DIP3;    %[s-1] rate of calcium diffusion

%% Additive wiener processes

N = 20; %number of IP3Rs in a cluster
s_IP3 = 0.02*ones(1,tot_cell_number);
s_Cac = 0.01*ones(1,tot_cell_number);
s_h = 0.07*ones(1,tot_cell_number);
s_Co = 0.01*ones(1,tot_cell_number);
s_o1 = 0*ones(1,tot_cell_number);
s_o2 = 0*ones(1,tot_cell_number);
s_c2 = 0*ones(1,tot_cell_number);
s_s = 0*ones(1,tot_cell_number);
SigX = [s_IP3, s_Cac, s_h, s_Co, s_o1, s_o2, s_c2, s_s];
SigX = SigX(:);

%% store all parameters
ParNames = who();
for i = 1:numel(ParNames)
    eval(['Param.' ParNames{i} '=' ParNames{i} ';' ]);
end

%% ---------------------------------------------------- initial conditions
% --------------- initial conditions
% ------SAM initial conditions
IP3_0 = 0.14*ones(1,tot_cell_number);  %uM
Ca_c_0 = 0.065*ones(1,tot_cell_number); %uM
h_0 = 0.79*ones(1,tot_cell_number); %dimless
co_0 = 2*ones(1,tot_cell_number); %uM

% ------ChR2 initial conditions
o1_in = 0*ones(1,tot_cell_number);
o2_in = 0*ones(1,tot_cell_number);
c2_in = 0*ones(1,tot_cell_number);
s_in = 0.1*ones(1,tot_cell_number);

X0 = [IP3_0, Ca_c_0, h_0, co_0, o1_in, o2_in,c2_in,s_in];
X0 = X0(:);

state_var_num = numel(X0);  % number of state variables

%%  Solve Usin LLScheme

nt = numel(tspan);
r = 1; % trapezoidal rule for integration using LL scheme
rng('shuffle')                    % shuffle random number generator
eta = randn(nt+1,state_var_num);  % system noise

tic
disp('solving....')
[ts,Xs] = SLLscheme(t0,X0,nt,dt,eta,'f','g','Jxf','Dft','Dgt','Hxf',r,Param); % LL scheme based solving of the equations
Xs = Xs';
toc

%% down sample for plotting
q = 1;      % increment for downsampling (1 == original signal)
tplot = ts(1:q:end);
Xplot = Xs(1:q:end,:);

%% state vars
ii=1;
IP3plot = Xplot(:,ii:tot_cell_number); 
Caplot = Xplot(:,ii*tot_cell_number+1:(ii+1)*tot_cell_number); ii =ii+1;
hplot = Xplot(:,ii*tot_cell_number+1:(ii+1)*tot_cell_number); ii =ii+1;
Coplot = Xplot(:,ii*tot_cell_number+1:(ii+1)*tot_cell_number); ii =ii+1;
o1plot = Xplot(:,ii*tot_cell_number+1:(ii+1)*tot_cell_number); ii =ii+1;
o2plot = Xplot(:,ii*tot_cell_number+1:(ii+1)*tot_cell_number); ii =ii+1;
c2plot = Xplot(:,ii*tot_cell_number+1:(ii+1)*tot_cell_number); ii =ii+1;
splot = Xplot(:,ii*tot_cell_number+1:(ii+1)*tot_cell_number);
C1plot = 1-o1plot-o2plot-c2plot; 

%% Ca traces
figure, plot(tplot,Caplot,'b'), hold on
plot(tplot, Caplot(:,laser_stim_cell_indx),'r','linewidth',1)
xlabel('time(s)'), ylabel('Ca(\muM)')
ax = gca;
ax.LineWidth = 2;
ax.FontSize = 16;
ax.Box = 'off';

%% Network visualization
ind1 = find(tplot<=laser_off-200,1,'last');
ind2 = find(tplot<=laser_off,1,'last');
basalCa = mean(Caplot(ind1:ind2,:),1);
basalCa = reshape(basalCa, cells_x, cells_y);

%% contour plot
[x,y] = meshgrid(1:cells_x, 1:cells_y);
[X,Y] = meshgrid(linspace(1,cells_x,10*cells_x), linspace(1,cells_y,10*cells_y));
basalCaq = interp2(x,y,basalCa',X,Y,'cubic');

figure, 
contourf(X,Y,basalCaq,30)
c = colorbar;
colormap jet
c.FontSize = 12;
ylabel(c, 'Ca_c(\muM)')

ax = gca;
ax.XTick = [];
ax.YTick = [];




