clear, clc, close all

% This code generates traces of Fig. 2 for the WT variant. 
% Please note that due to stochasticity, traces will not 
% be exactly the same as those in the paper.  

%% add required paths
addpath(genpath(fullfile(pwd,'../..')))

%% stimulation protocol
wiener_noise = true;
IP3_stim = false;
laser_stim = true;
buffer = true;

%% simulation time
t0 = 0;     %[s] start time
tf = 5000;  %[s] end time
dt = 0.05;   %[s] time increment
tspan = t0:dt:tf;    %[s] time span

%% laser stimulus
laser_on = 100;   %[s] stim onset
laser_off = tf;  %[s] stim end
T = 2;                %[s] pulse period
delta = 0.15*T;   %[s] pulse width

%% IP3 stimulus
IP3_stim_onset = 200;    %[s] onset of the IP3 stimulus
IP3_stim_end = 500;      %[s] end of IP3 stimulus
IP3_level_basal = 0.14;  % uM/s
IP3_level_stim = 0.4;       % factor of increase in IP3 level

%% ChR2 parameters (WT)
GChR2 = 0.1136;  %[mS/cm2]
p1 = 0.0641e3;    %s^-1; Max excitation rate of the 1st closed state
gd1 = 0.4558e3;   %s^-1; Rate const. for the o1-> c1 transition state
e12 = 0.2044e3;  %s^-1; Rate const for the o1->o2 transition
e21 = 0.009e3;    %s^-1; Rate const for the o2->o1 transition
p2 = 0.06102e3;    %s^-1; Max excitation rate of the 2nd closed state
gd2 = 0.0704e3;   %s^-1; Rate const for the o3-> c2 transition
gr = 9.3458e-5*1e3;  %s^-1; Recovery rate of the 1st closed state after light stimu is turned off
tau = 6.3152e-3;   %activation time of the ChR2 ion channel; gr = 1/tau
gamma = 0.0305;   %g2/g1 (ratio of conductances)
astro_area = 4.83e-6; %[cm^2] area of the astrocyte
Vm = -70;            %membrane voltage (mV)
vol_cyt = 1e-12; %[L] volume of the cytosol
F = 96485.3;    %[C/mol] faraday's constant
zCa = 2;    % valence of calcium
EChR2 = 0; %[mV] reversal potential of ChR2 (Williams et. al, 2013)
conversion_Fact = 1e-6/(zCa*F*vol_cyt);      % conversion factor for nA to uM/s
G_V = ((10.6408 - (14.6408*exp(-Vm/42.7671)))/Vm);
g1 = GChR2*astro_area*1e6;      % [nS] whole cell conductance of ChR2 channels in the cell (if current is in nA)

%% IP3 dynamics
vdelta = 0.152; % uM/s, Max. rate of IP3 production (PLC); Riera et al, Page 459,
KdCa = 0.55;    % uM; Half saturation const; Values: Fitting by Jorge Riera (PLCd)
KIP3 = 1.25;    % s^-1; IP3 degradation rate; Riera et al, 2011, Supp material for the SCOs in normal/ Alzheimers paper

%% Ca_C dynamics
xCCE = 0.01;    % uM/s; Max. rate of activation dependent Ca2+ influx (CCE); phenomological value; Riera et al - Page 450
hCCE = 10;      % uM; Half-inactivation constant for CCE influx; phenomological value; Riera et al - Page 450
c1 = 0.185;     % addimensional; Volume rate between ER and cytosol-> Ver/Vcyt; Values: 0.028-> DiGarbo, 0.05->Hofel
V_SERCA = 0.9;  % uM/sec; Max. rate const of SERCA pump (Max. Ca2+ pump flux); Riera et al - Page 447 - - check table for more value ranges Values = {0.9 uM/sec -> Goto&Natsume and Stamatakis&Mantzaris; 0.49 uM/sec -> Bennett}
Kp =  0.1;      % uM; Ca2+ sensitivity of the SERCA pump/ Dissociation const. of Ca2+ to SERCA; Riera et al - Page 447 - - check table for more value ranges; Values = {0.1 uM -> Goto&Natsume and Lavrentovich&Hemkin ;  0.05 uM Stamatakis&Mantzaris; 0.24 uM -> Bennett}
d1 =  0.13;     % uM;Dissocation constant for IP3(IP3R); Values = {0.13(d1),0.9434(d3)uM -> Goto&Natsume and Stamatakis&Mantzaris; 0.03uM -> Bennett}
d5 =  0.082;    % uM; Ca2+ activation constant (IP3R); Riera et al, 2011, Supp material for the SCOs in normal/ Alzheimers paper
v1 =  6;        % sec^(-1); Ca2+ channel flux const; Values = {6 sec^(-1) -> Goto&Natsume and Stamatakis&Mantzaris; 0.95 sec^(-1) -> Bennett
v2 = 0.11;      % sec^(-1); Ca2+ leak flux const; Goto&Natsume and Stamatakis&Mantzaris;
kout = 0.5;     % s^-1, Rate constant of Ca2+ extrusion; Riera et al, Page 451
lambda =  1;    % Riera et al SCO paper
epsilon = 0.01; % dimensionless, The ratio of plasma membrane and ER membrane areas (Li & Rinzel, page 468) e_m=0.01 Riera et al, 2011, Supp material for the SCOs in normal/ Alzheimers paper
Jin_Astro = 0.035;    % leak; uM/s, Riera et al, SCO paper-> Pg 449, used a heuristic strategy to determine jin -> 0:036 uM/s  for healthy astrocytes in acute slice preparations from adult C57BL/6 mice.

%% Gating parameters
a = 0.2;        %(uMs)^-1, Rate const for Ca2+ binding in IP3 inhibitory site; Riera et al, 2011, Supp material for the SCOs in normal/ Alzheimers paper
d2 = 1.049;     % uM; Dissociation const for Ca2+ inhibition (IP3R); Riera et al, 2011, Supp material for the SCOs in normal/ Alzheimers paper
d3 = 0.9434;    % uM; Dissociation const for IP3 (IP3R); Riera et al, 2011, Supp material for the SCOs in normal/ Alzheimers paper

%% buffering parameters
bt = 0; % uM  total buffer protein
K = 20; % uM

if buffer
    bt = 200; % uM
end

%% Additive wiener processes

N = 20; %number of IP3Rs in a cluster

s_IP3 = 0.02;
s_Cac = 0.01;
s_h = 0;
s_Co = 0.01;
s_o1 = 0;
s_o2 = 0;
s_c2 = 0;
s_s = 0;
SigX = [s_IP3, s_Cac, s_h, s_Co, s_o1, s_o2, s_c2, s_s];
SigX = SigX(:);

%% store all parameters
ParNames = who();
for i = 1:numel(ParNames)
    eval(['Param.' ParNames{i} '=' ParNames{i} ';' ]);
end

%% ---------------------------------------------------- initial conditions
% --------------- initial conditions
IP3_0 = 0.14;  %uM
Ca_c_0 = 0.09;%0.065; %uM
h_0 = 0.79; %dimless
co_0 = 2; %uM
o1_in = 0;
o2_in = 0;
c2_in = 0;
s_in = 0.1;

X0 = [IP3_0, Ca_c_0, h_0, co_0, o1_in, o2_in,c2_in,s_in];
X0 = X0(:);

state_var_num = numel(X0);  % number of state variables

%%  Solve Usin LLScheme

nt = numel(tspan);
r = 1; % trapezoidal rule for integration using LL scheme
rng('shuffle')                    % shuffle the rng seed
eta = randn(nt+1,state_var_num);  % system noise generation

tic
disp('solving....')
[ts,Xs] = SLLscheme(t0,X0,nt,dt,eta,'f','g','Jxf','Dft','Dgt','Hxf',r,Param); % LL scheme based solving of the equations
Xs = Xs';
toc

%% down sample for plotting
q = 1;      % increment for downsampling (1 == original signal)
tplot = ts(1:q:end);
Xplot = Xs(1:q:end,:);

%% state vars
i=1;
IP3plot = Xplot(:,i); i =i+1;
Caplot = Xplot(:,i); i =i+1;
hplot = Xplot(:,i); i =i+1;
Coplot = Xplot(:,i); i =i+1;
o1plot = Xplot(:,i); i =i+1;
o2plot = Xplot(:,i); i =i+1;
c2plot = Xplot(:,i); i =i+1;
splot = Xplot(:,i); i =i+1;
C1plot = 1-o1plot-o2plot-c2plot; 

%% plots

% ------------- Calcium
Fig = figure; 
plot(tplot,Caplot,'b','linewidth',0.5), hold on
xlabel('time(s)'), ylabel('[Ca^{2+}]_c(\muM)')
ax = gca;
ax.LineWidth = 2;
ax.FontSize = 16;
ax.FontName = 'arial';
ax.Box = 'off';
Fig.OuterPosition = [611, 550, 1166, 513];

% -------------- IP3
Fig = figure; 
plot(tplot,IP3plot,'b','linewidth',0.5), hold on
xlabel('time(s)'), ylabel('[IP3](\muM)')
ax = gca;
ax.LineWidth = 2;
ax.FontSize = 16;
ax.FontName = 'arial';
ax.Box = 'off';
Fig.OuterPosition = [611, 550, 1166, 513];

% --------------- h
Fig = figure; 
plot(tplot,hplot,'b','linewidth',0.5), hold on
xlabel('time(s)'), ylabel('h')
ax = gca;
ax.LineWidth = 2;
ax.FontSize = 16;
ax.FontName = 'arial';
ax.Box = 'off';
Fig.OuterPosition = [611, 550, 1166, 513];

% --------------- Co
Fig = figure; 
plot(tplot,Coplot,'b','linewidth',0.5), hold on
xlabel('time(s)'), ylabel('[C_o](\muM)')
ax = gca;
ax.LineWidth = 2;
ax.FontSize = 16;
ax.FontName = 'arial';
ax.Box = 'off';
Fig.OuterPosition = [611, 550, 1166, 513];


