function [t,z]=euler(t0,x0,n_m,deltat,eta,f,g,theta)
%   Euler scheme for the numerical integrator of the stochastic differential
% equations with multiplicative noise: 
%	  dx = f(t,x,theta) dt + g(t,x,theta) dw
%
% Notation
%   n_v_e: number of state variables x 
%
% Imputs:
%   t0:     initial time;
%   x0:     initial value of x at t0  (n_v_e x 1) 
%   n_m:    number of time-instants to integrate
%   deltat: integration step-size
%   eta:    matrix (n_v_e,n_m+1) with a sequence of random numbers N(0,I)
%   f:      string with the m file name of f
%   g:      string with the m file name of g, 
%   theta:  parameters of the differential equation
%
% Imputs of the above functions 
%   f(t,x,theta)           g(t,x,theta)
%
% Outputs:
%   t:   integrations times (1 x n_m)
%   z:   numerical solucion obtained by the LL scheme (n_v_e x n_m)

n_v_e = size(x0,1);
t = zeros(1,n_m);
z = zeros(n_v_e,n_m);

x = x0;
te = t0;

w=sqrt(deltat) .* cumsum([zeros(1,n_v_e);eta])';

t(1) = te;
z(:,1) = x;
for tt = 1:n_m-1
   fe = feval(f,te,x,theta);
   ge = feval(g,te,x,theta);
      
   m=size(ge,2);
   ruido=zeros(n_v_e,1);
   for k=1:m 
      ruido=ruido + ge{k}.*(w(k,tt+1)-w(k,tt));
   end;   
   te = tt .* deltat + t0;
   x = x + fe.*deltat + ruido;

   t(tt+1) = te;
   z(:,tt+1) = x;
   
end% for en tt

