% Script for computating the approximate innovation estimators and approximate fitting-filter 
% proposed in Jimenez (2012) for diffusion processes: 
%     dx = f(t,x,theta) dt + g(t,x,theta) dw, 
% given a set of discrete observations of some components of x. 

% These estimators for the uknown parameters theta and unobserved components of the process x are computed following 
% the heuristic introduced in Ozaki et. al (2000). This heuristic essential has three steps: a) computation of the 
% parameters theta; b) computation of the unobserved initial state of x; c) computation of the approxiamte-fitting 
% filter.
% For actual data a carefull selection of the aditional parameters: firstpoint, amplitude as well the options for the 
% optimization algorithm "fmincon" is very important to obtain successful results. In the same way an adequate 
% selection of the initial values for the parameters theta (theta0) and initial state (Xf0, Pf0) are crucial. 
% Thus, import modification must be done in the scripts for the generation and test that run before the present script.  
% Typically, before to run the present script many simulation with the scripts for generation and test must be done 
% to select adecuate values for the variables theta0, X0 and V0. 

% References:
%   -  "Approximate continuous-discrete filters for the estimation of diffusion processes 
%       from partial and noisy observations", (2012)
%       Jimenez J.C. http://arxiv.org/abs/1212.3721.
%   -  "An approximate innovation method for the estimation of diffusion processes from discrete data", 
%       Jimenez J.C. & Ozaki, J. Time Series Analysis 
%   -  "Role of the Likelihood Function in the Estimation of Chaos Models", 
%       Ozaki T., Jimenez J.C. & Haggab-Ozaki V., 
%       J. of Time Series Analysis (2000), Vol. 21, 363-387. 

echo off
%load

global deltat deltat_min n_v_e n_m n_obs t0 tTotal X0 Xf0 Pf0 mpts
global theta FileNames
global gct             % variable to be used in confun.m

%Definicion de todos las constantes
Const_Exac_Mul_NoL1;
Theta=theta;

%*********************  Estimation of the unknown model parameters (theta)
%this example shows the usefullness of the optimization with a priory statistics on the innovations
ch='P';
while ~(strcmp(ch,'Y') | strcmp(ch,'N'))
  ch=upper(input('Optimization with a priory statistics on the innovations? [y/n]= ','s'));
end;
if ch=='Y', 
  firstpoint=1;     % starting point for the computation of the innovations statistics
  lastpoint=n_m;    % last point for the computation of the innovations statistics
  nut=nu(:,firstpoint:lastpoint)';
  amplitude=1.2;     
  nu_stast=amplitude.*[abs(mean(nut)) diag(cov(nut))'];
else  
  nu_stast=[];
end

LB_T=[-0.15  0.05 -0.0015]; %0.5*theta
UB_T=[-0.05  0.15  0.0015]; %1.5*theta

LKWindow=[t0 tTotal];
Xf0Index=[];

ThetaIndex=[1 2];
LB=LB_T(ThetaIndex);
UB=UB_T(ThetaIndex);

Theta0=Theta;     
upper=1.1*Theta;
lower=0.9*Theta;
%Theta0(ThetaIndex)=(upper(ThetaIndex)-lower(ThetaIndex)).*rand(1,length(ThetaIndex)) + lower(ThetaIndex);

%Pf0 should be convenient adjuted if Xf0 are going to be estimated
OptimMethod='fmincon';
%OptimMethod='fminsearch';
%OptimMethod='patternsearch';
switch OptimMethod
  case 'fmincon'
    TolFunOpt=1e-9;                
    TolXOpt=1e-6;            
    OptimOptions = optimset('fmincon');
    OptimOptions = optimset(OptimOptions,'LargeScale','off','Display','iter','MaxFunEval',1e3,'TolFun',TolFunOpt,'TolX',TolXOpt);
  case 'fminsearch'
    TolFunOpt=1e-9;                
    TolXOpt=1e-6;            
    OptimOptions = optimset('fminsearch');
    OptimOptions = optimset(OptimOptions,'Display','iter','MaxFunEval',1e3,'TolFun',TolFunOpt,'TolX',TolXOpt);
  case 'patternsearch'
    TolFunOpt=1e-6;                
    TolXOpt=1e-2;            
    OptimOptions = psoptimset;
    %OptimOptions = psoptimset(OptimOptions,'PollMethod','MADSPositiveBasis2N','Display','iter','TolFun',TolFunOpt,'TolX',TolFunOpt);
    OptimOptions = psoptimset(OptimOptions,'Display','iter','TolFun',TolFunOpt,'TolX',TolFunOpt);
  otherwise
    error('Optimization Method is not well defined');    
end

EstimOptions = EstimSet(t,Z,Xf0,Pf0,Theta0,FileNames,LKWindow,mpts,nu_stast,LB,UB,OptimMethod,OptimOptions,AbsTol,RelTol);
[Xff,Pff,Xfp,Pfp,nu,LogLK_est,Xf0_Est,Theta_Est]=InnovEstimators(Xf0Index,ThetaIndex,EstimOptions);

disp(['   '])
disp([' Log LK for Theta_Est= ' num2str(-LogLK_est)])
disp(['   '])
disp(['        theta          theta0          theta_est'])
[Theta(ThetaIndex)' Theta0(ThetaIndex)' Theta_Est(ThetaIndex)']

% plot of the state variable and their filter estimates
figure
subplot(211)
plot(t,Z(1,:),t,var_est(1,:));
subplot(212)
plot(t,nu(1,:));

% Figure information
subplot(211), ylabel('x1'), title('simulated data vs. fitting-filter estimates')
subplot(212), title('fitting-innovation')

