function [Xf,Pf,Xp,Pp,Inn,LogLK,PInn,mpts]=LLfilter(t,Z,Xf0,Pf0,theta,FileNames,LKWindow,mpts,AbsTol,RelTol)
%    Local Linearization Filter for the estimation of the unobserved componet x of the 
% continuous model
%        dx = f(t,x,theta) dt + g(t,x,theta) dw, 
% with dicrete observation equation 
%         Z = h(t,x,theta) + p(t,x,theta)n + e, 
% given a set of discrete observation of Z. Xf0, Pf0 and theta are also known and given in advance.
%
% Notation
%   n_m: number of time-instants where Z is observed
%   n_obs: number of observation equations
%   n_v_e: number of state variables x 
%   n_theta: number of parameters theta 
%
% Input:
%   t: instants of times where at least one component of Z is observed  (1 x n_m)          
%   Z: observed sample                        (n_obs x n_m)   Set Z(k,t)=inf when data Z(k,t) is missing
%   Xf0: initial filter value                 (n_v_e x 1)
%   Pf0: initial filter variance value        (n_v_e x n_v_e)
%   theta: State Space Model parameters       (1 x n_theta)
%   FileNames: cell variable with the name of all the functions that define State Space Model
%   LKWindow: interval of time where the likelihood function will be computed  (1 x 2)
%   mpts: number of misssing points to compute the prediction between two observations (1 x n_m)
%   AbsTol,RelTol: absolute and relative tolerances for the adaptive LL filter (1 x 2)
%                  1: for the first moment, 2: for the second moment
%
%      FileNames={f,g,Jf,Jg,Hf,Hg,ft,gt,h,Jh,p,Jp,See,Lee}   
%       f: string with the m file name of drift coefficient f
%       g: string with the m file name of diffusion coefficient g
%      Jf: string with the m file name of the Jacobian of f 
%      Jg: string with the m file name of the Jacobian of g
%      Hf: string with the m file name of the Hessian of f 
%      Hg: string with the m file name of the Hessian of g
%      ft: string with the m file name of derivative of f respect to t
%      gt: string with the m file name of derivative of g respect to t
%       h: string with the m file name of h 
%       p: string with the m file name of p 
%      Jh: string with the m file name of the Jacobian of h 
%      Jp: string with the m file name of the Jacobian of p 
%     See: string with the m file name of the covariance matrix of the additive noise e 
%     Lee: string with the m file name of the covariance matrix of the multiplicative noise n 
%      ES: string with the m file name of the exact E(x/Zn) and E(xx'/Zn)
%
% OutPuts:
%   Xf : LL filter  
%   Pf : LL filter variance 
%   Xp : LL prediction 
%   Pp : LL prediction variance  
%   Inn: Innovation
%   LogLK:  -2log(Likelihood)
%   PInn: Innovation variance
%
%   [...] =  LLfilter(t,Z,Xf0,Pf0,theta,FileNames,LKWindow) compute the 
%   adaptive LL filter with the default absolute and relative tolerance 10^-6 and 10^-3
%   [...] =  LLfilter(t,Z,Xf0,Pf0,theta,FileNames,LKWindow,[],AbsTol,RelTol) compute the
%   adaptive LL filter with the specified absolute and relative tolerances AbsTol,RelTol
%   [...] =  LLfilter(t,Z,Xf0,Pf0,theta,FileNames,LKWindow,mpts) compute the 
%   non adaptive LL filter with the number of point between observations specified in input
%   variable mpts
%   [...] =  LLfilter(t,Z,Xf0,Pf0,theta,FileNames,LKWindow,[],0,0) compute the 
%   LL filter with exact predictions between observations specified in the file ES
%

%  References
%   - "Approximate linear minimum variance filters for continuous-discrete 
%     state space models: convergence and practical algorithms", 
%     Jimenez J.C.,    http://arxiv.org/abs/1207.6023 
%   - "Simplified formulas for the mean and variance of linear stochastic 
%     differential equations",  Jimenez J.C.,  http://arxiv.org/abs/1207.5067

if (nargin>=9) && (~isempty(AbsTol)), 
   if length(AbsTol)==2,
     Atol=AbsTol;     
   else
     Atol=[AbsTol,AbsTol];     
   end
else
   Atol=[1.0e-6,1.0e-6];
end

if (nargin==10) && (~isempty(RelTol)), 
   if length(RelTol)==2,
     Rtol=RelTol; 
   else
     Rtol=[RelTol,RelTol];     
   end
else
   Rtol=[1.0e-3,1.0e-3];
end

if nargin==7,
  [Xf,Pf,Xp,Pp,Inn,LogLK,PInn,mpts]=LLfilter_Adapt(t,Z,Xf0,Pf0,theta,FileNames,LKWindow,Atol,Rtol);
else
  if isempty(mpts),
      if Atol~=0,
        [Xf,Pf,Xp,Pp,Inn,LogLK,PInn,mpts]=LLfilter_Adapt(t,Z,Xf0,Pf0,theta,FileNames,LKWindow,Atol,Rtol);
      else
        [Xf,Pf,Xp,Pp,Inn,LogLK,PInn,mpts]=LLfilter_Exact(t,Z,Xf0,Pf0,theta,FileNames,LKWindow);
      end
  else
    [Xf,Pf,Xp,Pp,Inn,LogLK,PInn,mpts]=LLfilter_Mpts(t,Z,Xf0,Pf0,theta,FileNames,LKWindow,mpts);
  end
end
