function [ms, Ps, ms1, Ps1] = LLpredictionNoAutoAdd(s,A,f0,b,m0,P0,a1,b1)

% This function computes the conditional mean and variance of the equation:
%   dx = f(x,t)*dt + sum((b(t))*dw), x(0)=x0
% evaluated at the time s. 
% Here,f(x)=A*x+a(t), a(t)=a0*t+a1, b(t)=b0*t+b1, A is a constant matrix, and
% a0, a1, b0 and b1 are vectors
% These moments satisface the equations:
%   dm/dt = A*m+a(t),  m(0)=m0
%   dP/dt = A*P + P*A' + sum(b(t)*b'(t)), P(0)=P0
% In the definition of the function LLvariance the variable f0=f(0), i.e., 
% f0 = A*m0+a(t0).

%Reference
% "Local Linearization filters for non-linear continuous-discrete state 
%  space models with multiplicative noise", Jimenez J.C. & Ozaki T.
%  Int. J. Control (2003), 76, 1159-1170. 
% "Computing multiple integrals involving matrix exponentials"
%  Carbonell F & Jimenez J.C.

d=size(m0,1);
d2=d*d;

I_d=eye(d);
bbT=zeros(d,d);
b1bT=zeros(d,d);
bb1T=zeros(d,d);
b1b1T=zeros(d,d);
m=size(b,2);
for i=1:m 
   bbT = bbT + b{i}*b{i}';
   b1bT = b1bT + b1{i}*b{i}';
   bb1T = bb1T + b{i}*b1{i}';
   b1b1T= b1b1T + b1{i}*b1{i}';
end
a0=f0-A*m0;
v0=zeros(d,1);
LT=[I_d v0 v0];
R=[v0; 0; 1];

AE = kron(I_d,A) + kron(A,I_d);

beta1 = vec(bbT);
beta2 = vec(b1bT + bb1T);
beta3 = vec(b1b1T);
beta4 = kron(I_d,a0) + kron(a0,I_d);
beta5 = kron(I_d,a1) + kron(a1,I_d);

beta1 = beta1 + beta4*m0;
beta2 = beta2 + beta5*m0;
beta4 = beta4*LT;
beta5 = beta5*LT;

clear a0 v0 LT bbT b1bT bb1T

%sugestion: use krylov algorithm instead of expm if d>10

LC=d+2;
C=[A a1 f0; zeros(2,LC)];
C(d+1,LC)=1;
M1 = [AE beta5 beta4 beta3 beta2 beta1];
M2 = [zeros(LC,d2) C eye(LC) zeros(LC,3)];
M3 = [zeros(LC,d2+LC) C zeros(LC,3)];
M4 = [zeros(1,d2+2*LC) 0 2 0];
M5 = [zeros(1,d2+2*LC) 0 0 1];
EM = [M1; M2; M3; M4; M5; zeros(1,d2+2*LC+3)]; 

clear AE beta5 beta4 beta3 beta2 beta1 C M1 M2 M3 M4 M5

EM = expm(EM.*s);
Ps = EM(1:d2,1:d2)*vec(P0) + EM(1:d2,d2+LC+1:d2+2*LC)*R + EM(1:d2,end);
ms = m0 + EM(d2+1:d2+d,d2+d+2);
Ps = reshape(Ps,d,d);

if nargout>2,
  EM  = EM*EM;
  Ps1 = EM(1:d2,1:d2)*vec(P0) + EM(1:d2,d2+d+3:d2+d+2+LC)*R + EM(1:d2,end);
  ms1 = m0 + EM(d2+1:d2+d,d2+d+2);
  Ps1 = reshape(Ps1,d,d);
end  


%----------------------------------

function v = vec(x);

[n m]=size(x);
v=reshape(x,n*m,1);

