function [t,z] = SLLscheme(t0,x0,n_m,deltat,eta,f,g,Jfx,Jft,Jgt,Hfx,r,theta)
%   Local Linearization scheme for the numerical integrator of the stochastic differential
% equations with additive noise: 
%	  dx = f(t,x,theta) dt + g(t,theta) dw
%
% Notation
%   n_v_e: number of state variables x 
%
% Imputs:
%   t0:     initial time;
%   x0:     initial value of x at t0  (n_v_e x 1) 
%   n_m:    number of time-instants to integrate
%   deltat: integration step-size
%   eta:    matriz (n_v_e,n_m+1) con la secuencia  de numeros aleatorios N(0,I)
%   f:      string with the m file name of f
%   g:      string with the m file name of g, 
%   Jfx:    string with the m file name of the Jacobian of f 
%   Jft:    string with the m file name of the derivative of f respect to t
%   Jgt:    string with the m file name of the derivative of g respect to t
%   Hfx:    string with the m file name of the Hessian of f 
%     r:    trapezoidal rule {r=round(1/deltat) for compouse quadarature rule
%                             r=1;			  	   for simple quadrature rule
%   theta:  parameters of the differential equation
%
% Imputs of the above functions 
%   f(t,x,theta)           g(t,x,theta)
%   Jfx(t,x,theta)         Jgt(t,x,theta)
%   Jft(t,x,theta)         Hfx(t,x,theta)
%
% Outputs:
%   t:   integrations times (1 x n_m)
%   z:   numerical solucion obtained by the LL scheme (n_v_e x n_m)

%  References: 
%     - "Simulation of stochastic differential equations through the local 
%       linearization method. A comparative study". 
%       Jimenez, J.C., Shoji I, Ozaki T., J. Statist. Physics (1999), Vol. 94, 587-602 
%     - A simple algebraic expression to evaluate the Local Linearization schemes 
%       for stochastic differential equations, 
%       Jimenez, J.C., Appl. Math. Letters (2002), Vol. 15, 775-780. 


if strcmp(Jft,'zero') & strcmp(Jgt,'zero'),
   [t,z]=SLLscheme_Auto(t0,x0,n_m,deltat,eta,f,g,Jfx,Hfx,r,theta);
else
   [t,z]=SLLscheme_NoAuto(t0,x0,n_m,deltat,eta,f,g,Jfx,Jft,Jgt,Hfx,r,theta);
end  
   
   
%-----------------------------------------------------------------------------------

function [t,z] = SLLscheme_NoAuto(t0,x0,n_m,deltat,eta,f,g,Jfx,Jft,Jgt,Hfx,r,theta)

n_v_e = size(x0,1);
n_w   = size(eta,2);

t = zeros(1,n_m);
z = zeros(n_v_e,n_m);
ge= zeros(n_v_e,n_w);
g_te_deltat= zeros(n_v_e,n_w);
g_sigmak= zeros(n_v_e,n_w);
g_sigmak1= g_sigmak;
Jgt_sigmak= zeros(n_v_e,n_w);
Jgt_sigmak1= Jgt_sigmak;
x = x0;
te = t0;

w=sqrt(deltat/r) .* cumsum([zeros(1,n_w);eta])';

cont=1;
t(1) = te;
z(:,1) = x;
for tt = 1:n_m-1
   fe = feval(f,te,x,theta);
   Jfxe = feval(Jfx,te,x,theta);
 %  prueba(te,x,fe,Jfxe);                  %funcion para probar jacobiano de funciones complejas
   Jfte = feval(Jft,te,x,theta);
   
   ge_vector=feval(g,te,x,theta);
   ge_vector_deltat=feval(g,te+deltat,x,theta);
   for i=1:n_w
      ge(:,i)  = ge_vector{i};
      g_te_deltat(:,i) = ge_vector_deltat{i};
   end   
   geget = ge*ge';
  
   Ruido = g_te_deltat*w(:,cont+r) - expm(Jfxe.*deltat)*ge*w(:,cont);
   for k=0:r-1
  	 sigmak      = te + k*deltat/r;
	 sigmak1     = te + (k+1)*deltat/r;
	 phi_sigmak  = expm(Jfxe.*(te + deltat - sigmak));
     phi_sigmak1 = expm(Jfxe.*(te + deltat - sigmak1));
    
     ge_k  = feval(g,sigmak,x,theta);
     ge_k1 = feval(g,sigmak1,x,theta);
     Jge_k = feval(Jgt,sigmak,x,theta);
     Jge_k1= feval(Jgt,sigmak1,x,theta);
     for i=1:n_w
       g_sigmak(:,i)  = ge_k{i};
       g_sigmak1(:,i) = ge_k1{i};
       Jgt_sigmak(:,i) = Jge_k{i};
       Jgt_sigmak1(:,i) = Jge_k1{i};
     end  
   
     Zi1_sigmak  = phi_sigmak * (Jgt_sigmak - Jfxe * g_sigmak);
     Zi1_sigmak1 = phi_sigmak1 * (Jgt_sigmak1 - Jfxe * g_sigmak1);
	 Ruido = Ruido - (Zi1_sigmak1 * w(:,cont+1) + Zi1_sigmak * w(:,cont)).*deltat/(2*r);
     cont  = cont + 1;
   end;
   
   % for order 1.5 LL scheme
   S=zeros(n_v_e,1);
   if ( (r>1) & (~strcmp(Hfx,'zero')) ),
     Hfxe = feval(Hfx,te,x,theta);
     for i=1:n_v_e
       fila=n_v_e*(i-1);
       for j=1:n_w
         S=S+Hfxe(fila+1:fila+n_v_e,j).*geget(i,j);
       end
     end
     S=S./2;
   end;  
         
   te = tt .* deltat + t0;
   x = x + DLLscheme1(deltat,fe,Jfxe,Jfte+S) + Ruido;

   t(tt+1) = te;
   z(:,tt+1) = x;
     
end% for en tt


%-----------------------------------------------------------------------------------
function [t,z] = SLLscheme_Auto(t0,x0,n_m,deltat,eta,f,g,Jfx,Hfx,r,theta)

n_v_e = size(x0,1);
n_w   = size(eta,2);

t = zeros(1,n_m);
z = zeros(n_v_e,n_m);
ge= zeros(n_v_e,n_w);
g_te_deltat= zeros(n_v_e,n_w);

x = x0;
te = t0;

w=sqrt(deltat/r) .* cumsum([zeros(1,n_w);eta])';

cont=1;
t(1) = te;
z(:,1) = x;
for tt = 1:n_m-1
   fe = feval(f,te,x,theta);
   Jfxe = feval(Jfx,te,x,theta);
 %  prueba(te,x,fe,Jfxe);                  %funcion para probar jacobiano de funciones complejas
 
   ge_vector=feval(g,te,x,theta);
   ge_vector_deltat=feval(g,te+deltat,x,theta);
   for i=1:n_w
      ge(:,i)  = ge_vector{i};
      g_te_deltat(:,i) = ge_vector_deltat{i};
   end   
   geget = ge*ge';
  
   Ruido = g_te_deltat*w(:,cont+r) - expm(Jfxe.*deltat)*ge*w(:,cont);
   for k=0:r-1
  	  sigmak      = te + k*deltat/r;
	  sigmak1     = te + (k+1)*deltat/r;
	  phi_sigmak  = expm(Jfxe.*(te + deltat - sigmak));
     phi_sigmak1 = expm(Jfxe.*(te + deltat - sigmak1));
    
     ge_k  = feval(g,sigmak,x,theta);
     ge_k1 = feval(g,sigmak1,x,theta);
     for i=1:n_w
       g_sigmak(:,i)  = ge_k{i};
       g_sigmak1(:,i) = ge_k1{i};
     end  
   
     Zi1_sigmak  = - phi_sigmak * Jfxe * g_sigmak;
     Zi1_sigmak1 = - phi_sigmak1 * Jfxe * g_sigmak1;
	  Ruido = Ruido - (Zi1_sigmak1 * w(:,cont+1) + Zi1_sigmak * w(:,cont)).*deltat/(2*r);
     cont  = cont + 1;
   end;
   
   % for order 1.5 LL scheme
   S=zeros(n_v_e,1);
   if ( (r>1) & (~strcmp(Hfx,'zero')) ),
     Hfxe = feval(Hfx,te,x,theta);
     for i=1:n_v_e
       fila=n_v_e*(i-1);
       for j=1:n_w
         S=S+Hfxe(fila+1:fila+n_v_e,j).*geget(i,j);
       end
     end
     S=S./2;
   end;  
         
   te = tt .* deltat + t0;
   x = x + DLLscheme1(deltat,fe,Jfxe,S) + Ruido;

   t(tt+1) = te;
   z(:,tt+1) = x;
   
end% for en tt


%----------------------------------

function y = DLLscheme1(h,f,fx,ft);

n=size(f,1);
C=[fx, ft, f; zeros(2,n+2)];
C(n+1,n+2)=1;
M=expm(h*C);
y=M(1:n,n+2);


%----------------------------------
% test function for the exact Jacobian 

function prueba(te,x,fe,Jfxe);

d=length(x);
fac = [];
thresh = (1.e-3).*ones(d,1);
vectorized = 0;
[J,fac] = numjac('TestJacob',te,x,fe,thresh,fac,vectorized);
for i=1:d
   for j=1:d
      [i j Jfxe(i,j) J(i,j) Jfxe(i,j)-J(i,j)]
   end
end

%programa de prueba de DLLscheme

%x=randn(50,1);
%x1=randn(50,1);
%M=randn(50);
%h=randn;

%tic
%for i=1:100
%  n=size(M,1);
%  I=diag(ones(n,1));
%  E=expm(M*h);
%  Minv=M^(-1);
%  ro=Minv*(E-I);
%  r1=Minv*(E*h-ro);
%  LL=ro*x+(h*ro-r1)*x1;
%end
%toc

%tic
%for i=1:100
%   LL1=DLLscheme1(h,x,M,x1);
%end;   
%toc

%[LL LL1]

%la diferncia en tiempo es de 100seg el primero vs. 77seg el segundo

